@@ -434,8 +434,7 @@ event_db_list_for_period_internal (time_t start, time_t end, gboolean untimed,
   GSList *iter, *iter2;
   GSList *list = NULL;
   struct tm tm_event, tm_display;
-  long int check_recur_time, check_recur_month, check_recur_year;
-  long int time_increment=end-start;
+  long int month_diff;
 	  
   if (end == 0)		/* ??? pb */
     {
@@ -510,20 +509,30 @@ event_db_list_for_period_internal (time_t start, time_t end, gboolean untimed,
 	  abort ();
 	  
 	case RECUR_DAILY:
-	  if (alarms) check_recur_time = (start-event_start-ev->alarm)%(r->increment*SECONDS_IN_DAY);
-	  else check_recur_time = (start-event_start)%(r->increment*SECONDS_IN_DAY);
-	  if (check_recur_time < time_increment) 
+	  localtime_r (&start, &tm_display);
+          localtime_r (&event_start, &tm_event);
+	  if ((tm_event.tm_yday-tm_display.tm_yday)%r->increment==0) 
+	    {
+	      tm_event.tm_mon=tm_display.tm_mon;
+	      tm_event.tm_year=tm_display.tm_year;
+	      tm_event.tm_mday=tm_display.tm_mday;
+	    }; 
+	  clone_start=mktime(&tm_event);
+	  if (alarms) clone_start-=ev->alarm;
+	  if (clone_start <= end && (clone_start + ev->duration) > start) 
 	    {
 	      gboolean skip=FALSE;
+	      if (alarms) clone_start+=ev->alarm;
 	      if (r->exceptions) 
 	  	for (iter2 = r->exceptions; iter2; iter2 = g_slist_next (iter2))
-	  	  if ((long)iter2->data == (long)event_start) skip=TRUE;
+	  	  if ((long)iter2->data == (long)clone_start) skip=TRUE;
+	       if(alarms && start>clone_start-ev->alarm) skip=TRUE;
 	       if (!skip)
 	       {
 	  	  clone = event_db_clone(ev);
-	  	  if (alarms) clone->start = start-ev->alarm;
-	  	  else clone->start = start;
-	  	  clone->flags |= FLAG_RECUR;
+	  	  if (alarms) clone->start = clone_start-ev->alarm;
+	  	  else clone->start = clone_start;
+		  clone->flags |= FLAG_RECUR;
 	  	  list = g_slist_insert_sorted (list, clone, (GCompareFunc)event_sort_func);
 	       }
 	    }
@@ -532,52 +541,59 @@ event_db_list_for_period_internal (time_t start, time_t end, gboolean untimed,
 	case RECUR_WEEKLY:
           localtime_r (&start, &tm_display);
           localtime_r (&event_start, &tm_event);
-	  if (alarms) check_recur_time = (start-event_start-ev->alarm)%SECONDS_IN_DAY;
-	  else check_recur_time = (start-event_start)%SECONDS_IN_DAY;
 	  if (((r->daymask & MON && tm_display.tm_wday==1) ||
-	       (r->daymask & TUE && tm_display.tm_wday==2) ||
-	       (r->daymask & WED && tm_display.tm_wday==3) ||
-	       (r->daymask & THU && tm_display.tm_wday==4) ||
-	       (r->daymask & FRI && tm_display.tm_wday==5) ||
-	       (r->daymask & SAT && tm_display.tm_wday==6) ||
-	       (r->daymask & SUN && tm_display.tm_wday==0)) &&
-	       check_recur_time < time_increment) {
+	     (r->daymask & TUE && tm_display.tm_wday==2) ||
+	     (r->daymask & WED && tm_display.tm_wday==3) ||
+	     (r->daymask & THU && tm_display.tm_wday==4) ||
+	     (r->daymask & FRI && tm_display.tm_wday==5) ||
+	     (r->daymask & SAT && tm_display.tm_wday==6) ||
+	     (r->daymask & SUN && tm_display.tm_wday==0))) {
+	    tm_event.tm_mon=tm_display.tm_mon;
+	    tm_event.tm_year=tm_display.tm_year;
+	    tm_event.tm_mday=tm_display.tm_mday;
+	    clone_start=mktime(&tm_event);
+	    if (alarms) clone_start-=ev->alarm;
+	    if (clone_start <= end && (clone_start + ev->duration) > start) 
+	    {
 	      gboolean skip=FALSE;
+	      if (alarms) clone_start+=ev->alarm;
 	      if (r->exceptions) 
-	  	for (iter2 = r->exceptions; iter2; iter2 = g_slist_next (iter2))
-	  	  if ((long)iter2->data == (long)event_start) skip=TRUE;
-	       tm_display.tm_hour=tm_event.tm_hour;
-	       tm_display.tm_min=tm_event.tm_min;
-	       tm_display.tm_sec=tm_event.tm_sec;
-	       clone_start=mktime(&tm_display);
-	       if(alarms && start>clone_start-ev->alarm) skip=TRUE;
-	       if (!skip)
-	       {
-	  	  clone = event_db_clone(ev);
-	  	  if (alarms) clone->start = clone_start-ev->alarm;
-	  	  else clone->start = clone_start;
-	  	  clone->flags |= FLAG_RECUR;
-	  	  list = g_slist_insert_sorted (list, clone, (GCompareFunc)event_sort_func);
-	       }
-	  }
+	    	for (iter2 = r->exceptions; iter2; iter2 = g_slist_next (iter2))
+	    	  if ((long)iter2->data == (long)clone_start) skip=TRUE;
+	      if(alarms && start>clone_start-ev->alarm) skip=TRUE;
+	      if (!skip)
+	    	{
+	    	  clone = event_db_clone(ev);
+	    	  if (alarms) clone->start = clone_start-ev->alarm;
+	    	  else clone->start = clone_start;
+	    	  clone->flags |= FLAG_RECUR;
+	    	  list = g_slist_insert_sorted (list, clone, (GCompareFunc)event_sort_func);
+	    	}
+	      }
+	    }
 	  break;
 
 	case RECUR_MONTHLY:
           localtime_r (&start, &tm_display);
           localtime_r (&event_start, &tm_event);
-	  if (alarms) check_recur_time = (start-event_start-ev->alarm)%SECONDS_IN_DAY;
-	  else check_recur_time = (start-event_start)%SECONDS_IN_DAY;
-	  check_recur_month = (tm_display.tm_mon-tm_event.tm_mon)%(r->increment);
-	  if (tm_display.tm_mday==tm_event.tm_mday && check_recur_time < time_increment
-	        && check_recur_month==0) {
+	  month_diff=tm_display.tm_mon-tm_event.tm_mon +
+	   (tm_display.tm_year-tm_event.tm_year)*12;
+	  if ((tm_display.tm_mday==tm_event.tm_mday) &&
+	   (month_diff%r->increment)==0) 
+	    {
+	      tm_event.tm_mon=tm_display.tm_mon;
+	      tm_event.tm_year=tm_display.tm_year;
+	      tm_event.tm_mday=tm_display.tm_mday;
+	    }; 
+	  clone_start=mktime(&tm_event);
+	  if (alarms) clone_start-=ev->alarm;
+	  if (clone_start <= end && (clone_start + ev->duration) > start) 
+	    {
 	      gboolean skip=FALSE;
+	      if (alarms) clone_start+=ev->alarm;
 	      if (r->exceptions) 
 	  	for (iter2 = r->exceptions; iter2; iter2 = g_slist_next (iter2))
-	  	  if ((long)iter2->data == (long)event_start) skip=TRUE;
-	       tm_display.tm_hour=tm_event.tm_hour;
-	       tm_display.tm_min=tm_event.tm_min;
-	       tm_display.tm_sec=tm_event.tm_sec;
-	       clone_start=mktime(&tm_display);
+	  	  if ((long)iter2->data == (long)clone_start) skip=TRUE;
 	       if(alarms && start>clone_start-ev->alarm) skip=TRUE;
 	       if (!skip)
 	       {
@@ -593,19 +609,23 @@ event_db_list_for_period_internal (time_t start, time_t end, gboolean untimed,
 	case RECUR_YEARLY:
           localtime_r (&start, &tm_display);
           localtime_r (&event_start, &tm_event);
-	  if (alarms) check_recur_time = (start-event_start-ev->alarm)%SECONDS_IN_DAY;
-	  else check_recur_time = (start-event_start)%SECONDS_IN_DAY;
-	  check_recur_year = (tm_display.tm_year-tm_event.tm_year)%(r->increment);	  
-          if (tm_display.tm_mon==tm_event.tm_mon && check_recur_time < time_increment &&
-	        tm_display.tm_mday==tm_event.tm_mday && check_recur_year==0) {
+	  if ((tm_display.tm_mon==tm_event.tm_mon) &&
+	   (tm_display.tm_mday==tm_event.tm_mday) &&
+	   ((tm_display.tm_year-tm_event.tm_year)%r->increment)==0) 
+	    {
+	      tm_event.tm_mon=tm_display.tm_mon;
+	      tm_event.tm_year=tm_display.tm_year;
+	      tm_event.tm_mday=tm_display.tm_mday;
+	    }; 
+	  clone_start=mktime(&tm_event);
+	  if (alarms) clone_start-=ev->alarm;
+	  if (clone_start <= end && (clone_start + ev->duration) > start) 
+	    {
 	      gboolean skip=FALSE;
+	      if (alarms) clone_start+=ev->alarm;
 	      if (r->exceptions) 
 	  	for (iter2 = r->exceptions; iter2; iter2 = g_slist_next (iter2))
-	  	  if ((long)iter2->data == (long)event_start) skip=TRUE;
-	       tm_display.tm_hour=tm_event.tm_hour;
-	       tm_display.tm_min=tm_event.tm_min;
-	       tm_display.tm_sec=tm_event.tm_sec;
-	       clone_start=mktime(&tm_display);
+	  	  if ((long)iter2->data == (long)clone_start) skip=TRUE;
 	       if(alarms && start>clone_start-ev->alarm) skip=TRUE;
 	       if (!skip)
 	       {
@@ -683,7 +703,18 @@ event_db_list_for_future (time_t start, guint max)
   int NUMBER_OF_DAYS=14;
   int day_inc;
   GSList *return_list = NULL;
+  time_t shifted_end;
+  struct tm tm_shift;
   
+  localtime_r (&start, &tm_shift);
+  tm_shift.tm_mday++;
+  tm_shift.tm_hour=0;
+  tm_shift.tm_min=0;
+  tm_shift.tm_sec=1;
+  shifted_end=mktime(&tm_shift);
+  return_list=event_db_list_for_period_internal (start, shifted_end, FALSE, FALSE, TRUE, 0);
+  start=shifted_end;
+   
   for (day_inc=0;day_inc<NUMBER_OF_DAYS;day_inc++) {
   	return_list = g_slist_concat(return_list, event_db_list_for_period_internal (start,
 	  start+SECONDS_IN_DAY, FALSE, FALSE, FALSE, max));
